--- //depot/vendor/freebsd/src/sys/kern/kern_acct.c	2007/06/05 00:03:37
+++ //depot/user/attilio/attilio_schedlock/kern/kern_acct.c	2007/06/08 13:38:46
@@ -370,8 +370,7 @@
 	bcopy(p->p_comm, acct.ac_comm, sizeof acct.ac_comm);
 
 	/* (2) The amount of user and system time that was used */
-	rufetch(p, &ru);
-	calcru(p, &ut, &st);
+	rufetchcalc(p, &ru, &ut, &st);
 	acct.ac_utime = encode_timeval(ut);
 	acct.ac_stime = encode_timeval(st);
 
--- //depot/vendor/freebsd/src/sys/kern/kern_resource.c	2007/06/05 00:03:37
+++ //depot/user/attilio/attilio_schedlock/kern/kern_resource.c	2007/06/08 13:38:46
@@ -1056,6 +1056,54 @@
 }
 
 /*
+ * Perform an atomic rufetch() and calcru() together.
+ */
+void
+rufetchcalc(struct proc *p, struct rusage *ru, struct timeval *up,
+    struct timeval *sp)
+{
+	struct rusage_ext rux;
+	struct thread *td;
+	uint64_t u;
+
+	PROC_LOCK_ASSERT(p, MA_OWNED);
+
+	memset(ru, 0, sizeof(*ru));
+	PROC_SLOCK(p);
+	if (p->p_ru == NULL)  {
+		KASSERT(p->p_numthreads > 0,
+		    ("rufetch: No threads or ru in proc %p", p));
+		FOREACH_THREAD_IN_PROC(p, td) {
+			thread_lock(td);
+			ruxagg(&p->p_rux, td);
+			thread_unlock(td);
+			rucollect(ru, &td->td_ru);
+		}
+	} else
+		*ru = *p->p_ru;
+	/*
+	 * If we are getting stats for the current process, then add in the
+	 * stats that this thread has accumulated in its current time slice.
+	 * We reset the thread and CPU state as if we had performed a context
+	 * switch right here.
+	 */
+	td = curthread;
+	if (td->td_proc == p) {
+		u = cpu_ticks();
+		p->p_rux.rux_runtime += u - PCPU_GET(switchtime);
+		PCPU_SET(switchtime, u);
+	}
+	/* Work on a copy of p_rux so we can let go of p_slock */
+	rux = p->p_rux;
+	PROC_SUNLOCK(p);
+	calcru1(p, &rux, up, sp);
+	/* Update the result from the p_rux copy */
+	p->p_rux.rux_uu = rux.rux_uu;
+	p->p_rux.rux_su = rux.rux_su;
+	p->p_rux.rux_tu = rux.rux_tu;
+}
+
+/*
  * Allocate a new resource limits structure and initialize its
  * reference count and mutex pointer.
  */
--- //depot/vendor/freebsd/src/sys/sys/resourcevar.h	2007/06/05 00:03:37
+++ //depot/user/attilio/attilio_schedlock/sys/resourcevar.h	2007/06/08 13:50:53
@@ -123,8 +123,10 @@
 void	 lim_rlimit(struct proc *p, int which, struct rlimit *rlp);
 void	 ruadd(struct rusage *ru, struct rusage_ext *rux, struct rusage *ru2,
 	    struct rusage_ext *rux2);
+void	 rucollect(struct rusage *ru, struct rusage *ru2);
 void	 rufetch(struct proc *p, struct rusage *ru);
-void	 rucollect(struct rusage *ru, struct rusage *ru2);
+void	 rufetchcalc(struct proc *p, struct rusage *ru, struct timeval *up,
+	    struct timeval *sp);
 void	 ruxagg(struct rusage_ext *rux, struct thread *td);
 int	 suswintr(void *base, int word);
 struct uidinfo
